/*
 *    MessageClassifier.java
 *    Copyright (C) 1999 Stuart Inglis,Len Trigg,Eibe Frank
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
// 02.10.2001

//no here     I have changed in the program weka\core\Utils.java
//    /** The small deviation allowed in double comparisons */
//   // public static double SMALL = 1e-6;
//    public static final double SMALL = 1e-6;

/**
 * Java program for classifying short text messages into two classes.
 */
import weka.core.*;
import weka.classifiers.*;
import java.io.*;
import java.util.*;

public class MessageClassifier {
    public static final String[] m_Keywords = { "a" ,
                                                "b" ,
                                          //    "c" ,
                                          //    "d" ,
                                          //    "e" ,
                                          //    "v" ,
                                          //    "w" ,
                                          //    "x" ,
                                                "y" ,
                                                "z" };

    public static final String[][] textLearn = { { "a", "hit"  },
                                                 { "b", "hit"  },
                                           //    { "c", "hit"  },
                                           //    { "d", "hit"  },
                                           //    { "e", "hit"  },
                                           //    { "v", "miss" },
                                           //    { "w", "miss" },
                                           //    { "x", "miss" },
                                                 { "y", "miss" },
                                                 { "z", "miss" } }; 
    public static String textPredict = "z";

    public Instances m_Data = null;

    /**
     * Classification of the textPredict depending on the textLearn  
     */
    public static void main(String[] options) throws Exception{
    //Id3   IBk  SMO.java 
        Classifier m_Classifier = new SMO();
        MessageClassifier messageCl = new MessageClassifier();

        for  (int i = 0; i < textLearn.length; i++) {
            messageCl.updateModel(m_Classifier,
                                  textLearn[i][0],
                                  textLearn[i][1]);
        }

        String classified =  messageCl.classifyMessage(m_Classifier,
                                                       textPredict);
        System.out.println("Message classified as : " + classified);
    }

    /**
     * Constructs empty training dataset.
     */
    public MessageClassifier() throws Exception {
        String nameOfDataset = "MessageClassificationProblem";
            // Create numeric attributes.
        FastVector attributes = new FastVector(m_Keywords.length + 1);
        for (int i = 0 ; i < m_Keywords.length; i++) {
            attributes.addElement(new Attribute(m_Keywords[i]));
        }
            // Add class attribute.
        FastVector classValues = new FastVector(2);
        classValues.addElement("miss");
        classValues.addElement("hit");
        attributes.addElement(new Attribute("Class", classValues));
            // Create dataset with initial capacity of 100, and set index of class.
        m_Data = new Instances(nameOfDataset, attributes, 100);
        m_Data.setClassIndex(m_Data.numAttributes() - 1);
   }

    /**
     * Updates model using the given training message.
     */
    public void updateModel(Classifier m_Classifier,
                            String message,
                            String classValue) throws Exception {
            // Convert message string into instance.
        Instance instance = makeInstance(message);
            // Add class value to instance.
        instance.setClassValue(classValue);
            // Add instance to training data.
        m_Data.add(instance);
        m_Classifier.buildClassifier(m_Data);
    }

    /**
     * Classifies a given message.
     */
    public String classifyMessage(Classifier m_Classifier,
                                  String message) throws Exception {
            // Convert message string into instance.
        Instance instance = makeInstance(message);
            // Get index of predicted class value.
        double predicted = m_Classifier.classifyInstance(instance);
            // Classify instance.
        String classified = m_Data.classAttribute().value((int)predicted);
        return classified;
    }

    /**
     * Method that converts a text message into an instance.
     */
    public Instance makeInstance(String messageText) {
        Instance instance = new Instance(m_Keywords.length + 1);
        for (int i = 0; i < m_Keywords.length; i++) {
            if (messageText == m_Keywords[i]) {
                      instance.setValue(i, 1.0);}
            else {    instance.setValue(i, 0.0);}
        }
            // Give instance access to attribute information from the dataset.
        instance.setDataset(m_Data);
        return instance;
    }

}