package weka.classifiers;

import java.io.*;
import java.util.*;
import weka.core.*;


/**
 * IB1-type classifier. Uses a simple distance measure to find the training
 * instance closest to the given test instance
 */
public class IBKorl extends Classifier {

//   public static Instances m_Train = null;
   public Instances m_Train = null;


  /**
   * Generates the classifier.
   * @param instances set of instances serving as training data 
   * @exception Exception if the classifier has not been generated successfully
   */
  public final void buildClassifier(Instances instances) throws Exception{
     m_Train = new Instances(instances, 0, instances.numInstances());
     Enumeration enum = m_Train.enumerateInstances();
  }


  /**
   * Classifies the given test instance.
   *
   * @param instance the instance to be classified
   * @return the predicted class for the instance 
   * @exception Exception if the instance can't be classified
   */
  public final double classifyInstance(Instance instance)throws Exception {

    double distance;
    Enumeration enum = m_Train.enumerateInstances();
    double classValue = 0.0;
    
    while (enum.hasMoreElements()) {
        Instance trainInstance = (Instance) enum.nextElement();
        distance = distance(instance, trainInstance);
        if (distance == 0.0) {
            classValue = trainInstance.classValue();
        }
    }
    return classValue;
  }

  /**
   * Calculates the distance between two instances
   *
   * @param first the first instance
   * @param second the second instance
   * @return the distance between the two given instances
   */          
  public final double distance(Instance first, Instance second)
  throws Exception {
      double distance = 0.0;

      for(int i = 0; i < m_Train.numAttributes() -1; i++) {
          if (first.value(i) != second.value(i)) {
              distance = 1.0;
          }
	  }
      return distance;
  }

}